from Products.PythonScripts import PythonScript
from Globals import DTMLFile, package_home
from urllib import quote
import os

# The Zope product object inherits all of what it does
# from a normal Zope PythonScript object. We even use
# the web forms from the other product as well. This
# means all the security and persistence comes for free.
# The reason that a new type is introduced is so that
# we can distinguish between normal Zope python scripts
# and those we want to be run when Zope starts. Note
# that only instances of the startup script which are
# in the root folder are run. This is to avoid traversing
# the whole ZODB which could be time consuming.

class StartupScript(PythonScript.PythonScript):
  meta_type = 'Startup Script (Python)'

manage_addPythonScriptForm = DTMLFile(
 '../PythonScripts/www/pyScriptAdd', globals())

_default_file = os.path.join(package_home(globals()),
 'www', 'default_py')

# The method for adding in an instance of our startup
# script object is the only method which really needs
# to be redefined. This is because we need to create
# an instance of StartupScript instead of PythonScript.

def manage_addPythonScript(self, id, REQUEST=None, submit=None):
  """Add a Python script to a folder.
  """
  id = str(id)
  id = self._setObject(id, StartupScript(id))
  if REQUEST is not None:
    file = REQUEST.form.get('file', '')
    if type(file) is not type(''): file = file.read()
    if not file:
	file = open(_default_file).read()
    self._getOb(id).write(file)
    try: u = self.DestinationURL()
    except: u = REQUEST['URL1']
    if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
    REQUEST.RESPONSE.redirect(u+'/manage_main')
  return ''

# This is where the magic occurs. This method will be
# called immediately upon asyncore entering its main
# loop. This will be prior to any web requests being
# serviced. Note however that if any scripts called
# block, this will stall the startup process of Zope
# until the scripts complete.

def startup_PythonScripts(*args):
  from zLOG import LOG, WARNING, INFO, ERROR, DEBUG
  from AccessControl import SecurityManagement
  import sys

  # We need to get the application context from some
  # place and there doesn't seem to be a properly
  # defined way of doing this. We grab it from the
  # "Zope" module, although this isn't going to work
  # if Zope is run up against the "Main" module
  # instead. Although we could grab the context from
  # the product context provided at time of product
  # initialisation, we need to break encapsulation to
  # do this and the jar associated with that context
  # gets closed off before we get to use it in our
  # callback from asyncore. If there is a better way
  # of getting the application context, please let
  # us know.

  from Zope import bobo_application
  c = bobo_application()

  # Go through each object in the root folder and if
  # it is a startup script, execute it with the
  # privileges of the owner. If a script which is
  # not in the root folder needs to be run, call
  # that script from one of the startup scripts in
  # the root folder. Any output from the scripts,
  # or error messages will be logged. If an error
  # does occur, that error will be ignored and the
  # processing of other scripts will continue.

  for id,object in c.objectItems(StartupScript.meta_type):
    owner = object.getOwner()
    SecurityManagement.newSecurityManager(None,owner)

    try:
      result = str(object())
      if result == '' or result[-1] != '\n':
	result += '\n'
    except:
      LOG(StartupScript.meta_type,ERROR,"at /%s, owner=%s, failed." % \
       (object.getPhysicalPath()[1],owner),error=sys.exc_info())
    else:
      LOG(StartupScript.meta_type,INFO,"at /%s, owner=%s, started." % \
       (object.getPhysicalPath()[1],owner),result)

    SecurityManagement.noSecurityManager()

  # Close off the jar and try and dispose of the
  # application context.

  c._p_jar.close()
  del c
